local addonName, at = ...

at.Abet = {}
local ipairs, pairs, stringFind, stringGMatch = ipairs, pairs, string.find, string.gmatch

function at.Abet:FindPattern(var, pat)
	local val = ""
	for word in string.gmatch(var, pat) do
		val = val .. word
	end
	return val
end
function at.Abet:CopyTable(t)
	if not t then return {} end
	local t2 = {}
	for k,v in pairs(t) do
		if type(v) == 'table' then
			v = self:CopyTable(v)
		end
		t2[k] = v
	end
	return t2
end
function at.Abet:ExistsAsTableKey(aTable, var)
	if not var then return false end
	for x, v in pairs(aTable) do
		if var == x then return true end
	end
	return false
end
function at.Abet:ExistsInTable(aTable, var)
	if not var then return false end
	for x, v in pairs(aTable) do
		if var == v then return true end
	end
	return false
end
function at.Abet:ExistsInTableMatch(aTable, var)
	if not var then return false end
	for x, v in pairs(aTable) do
		if var:find(v) then return true end
	end
	return false
end
function at.Abet:ExistsInTablePattern(aTable, var)
	if not var then return false end
	for x, v in pairs(aTable) do
		if stringFind(name, at.Abet:FindPattern(v, "%u")) then return true end
	end
	return false
end
function at.Abet:ExistsInTablePatternMatch(aTable, var)
	if not var then return false end
	for x, v in pairs(aTable) do
		if var == v or var:find(v) or stringFind(name, at.Abet:FindPattern(v, "%u")) then return true end
	end
	return false
end
function at.Abet:GetTableValueIndex(aTable, var)
	if not var then return 0 end
	for x, v in ipairs(aTable) do
		if var == v then return x end
	end
	return 0
end
function at.Abet:DeleteFromTable(aTable, var)
	if not var then return {} end
	for x, v in ipairs(aTable) do
		if x == var then table.remove(aTable, x) break end
	end
	return aTable
end
function at.Abet.GetRelativePoint(frame)
	local L1, B1, W1, H1 = UIParent:GetRect()
	local L2, B2, W2, H2 = frame:GetRect()
	local x, y, h, w, anchor = L2 - L1, B2 - B1, H1 - H2, W1 - W2
	if x < w * 0.5 then
		anchor = 'LEFT'
	else
		anchor, x = 'RIGHT', x - w
	end
	if y < h * 0.5 then
		anchor = 'BOTTOM' .. anchor
	else
		anchor, y = 'TOP' .. anchor, y - h
	end
	return anchor, x, y
end
function at.Abet:GetFont(name)
	if not not LibStub then
		local sub = LibStub.libs['LibSharedMedia-3.0']
		if not not sub then
			return sub:Fetch('font', name:gsub("_", " "))
		end
	end
	return at.Cache.Fonts[name:gsub("_", " ")]
end
function at.Abet:ListFonts()
	print('Name : Source')
	if not not LibStub then
		local sub = LibStub.libs['LibSharedMedia-3.0']
		if not not sub then
			for x, v in pairs(sub:List('font')) do
				print("  " .. v:gsub(" ", "_") .. ' :\n   ' .. sub:Fetch('font', v))
			end
			return
		end
	end
	for x, v in pairs(at.Cache.Fonts) do
		print(x .. ' : ' .. v)
	end
end
function at.Abet:FormatTime(aTime)
	local txt, hours, minutes, seconds = '', 0, 0, 0

	days = math.floor( aTime /3600 /24 )--math.floor isn't required but makes me feel safe
	hours = math.floor( aTime /3600 )
	minutes = math.floor( aTime %3600 /60 )
	seconds = math.floor( aTime %3600 %60 )

	if days > 0 then txt = txt .. string.format(at.Global.D_DAYS, days) .. " " end
	if hours > 0 then txt = txt .. string.format(at.Global.D_HOURS, hours) .. " " end
	if minutes > 0 then txt = txt .. string.format(at.Global.D_MINUTES, minutes) .. " " end
	if seconds > 0 then txt = txt .. string.format(at.Global.D_SECONDS, seconds) .. " " end
	return txt
end
function at.Abet:IsInCombat() return at.Cache.InCombat end
