local addonName, at = ...

at.Core = CreateFrame("Frame", addonName .. "CoreFrame", UIParent)
at.Events = {}
at.Settings = {}
local addonDir = [[Interface\AddOns\]] .. addonName
at.Cache = {
	InCombat = InCombatLockdown(),
	Directory = addonDir,
	Version = {
		number = GetAddOnMetadata(addonName, "Version"),
		ResetLevel = 0,
	},
	Minimap = {
		Icons = {
			MiniMapMailFrame = 'BOTTOMRIGHT',
			MiniMapVoiceChatFrame = 'BOTTOMLEFT',
		},
		Blip = {
			blink = false,
			Icons = {
				default = [[Interface\Minimap\ObjectIcons]],
				blit	= addonDir .. [[\media\Blip\ObjIcos]],
				herb	= addonDir .. [[\media\Blip\ObjIcos_Herb]],
				mine	= addonDir .. [[\media\Blip\ObjIcos_Mine]],
				both	= addonDir .. [[\media\Blip\ObjIcos_Mix]],
			},
		},
		Shape = {
			dir = addonDir .. [[\media\Mask\]],
			list = {
				'ROUND',--standard shape
				'SQUARE',--'nuff said
				'CORNER-TOPLEFT',--top left corner is round
				'CORNER-TOPRIGHT',--top right corner is round
				'CORNER-BOTTOMLEFT',--bottom left corner is round
				'CORNER-BOTTOMRIGHT',--bottom right corner is round
				'SIDE-LEFT',--left side is square
				'SIDE-RIGHT',--right side is square
				'SIDE-TOP',--top is square
				'SIDE-BOTTOM',--bottom is square
				'TRICORNER-TOPLEFT',--all round but top left
				'TRICORNER-TOPRIGHT',--all round but top right
				'TRICORNER-BOTTOMLEFT',--all round but bottom left
				'TRICORNER-BOTTOMRIGHT',--all round but bottom right
			},
		},
	},
	Fonts = {
		['Arial Narrow']		= [[Fonts\ARIALN.TTF]],
		['Friz Quadrata']		= [[Fonts\FRIZQT__.TTF]],
		['Morpheus']			= [[Fonts\MORPHEUS.TTF]],
		['Skurri']				= [[Fonts\SKURRI.TTF]],
	},
	Arrow = {
		JMMArrow	= addonDir .. [[\media\TomTomLite\BevArrow]],
		JMMGlow		= addonDir .. [[\media\TomTomLite\BevArrowGlow]],
	},
	Addons = {
		directions = {
			['DOWN'] = {'TOP', 'BOTTOM',},
			['UP'] = {'BOTTOM', 'TOP',},
			['RIGHT'] = {'LEFT', 'RIGHT',},
			['LEFT'] = {'RIGHT', 'LEFT',},
			['UPRIGHT'] = {'BOTTOMLEFT', 'TOPRIGHT'},
			['UPLEFT'] = {'BOTTOMRIGHT', 'TOPLEFT'},
			['DOWNRIGHT'] = {'TOPLEFT', 'BOTTOMRIGHT'},
			['DOWNLEFT'] = {'TOPRIGHT', 'BOTTOMLEFT'},
		},
		capture = {},
	},
}
at.Global = {
	D_DAYS = D_DAYS,
	D_HOURS = D_HOURS,
	D_MINUTES = D_MINUTES,
	D_SECONDS = D_SECONDS,
	TANK = TANK,
	HEALER = HEALER,
	DAMAGE = DAMAGE,
	TIME_IN_QUEUE = TIME_IN_QUEUE,
	AVERAGE_WAIT_TIME = AVERAGE_WAIT_TIME,
}


at.Core:RegisterEvent('ADDON_LOADED')
--ADDON_LOADED; fires for all addons being loaded
function at.Events:ADDON_LOADED(aName, ...)
	if aName ~= addonName then return else end--so skip over addons until it loads this one
	self:UnregisterEvent('ADDON_LOADED')
	at.Events['ADDON_LOADED'] = nil

	for k, v in pairs(at.Events) do
		if k ~= 'ADDON_LOADED' then
			self:RegisterEvent(k)--Register all other events for which handlers have been defined
		end
	end

	if not JMMDB then
		at.Cache.Version.ResetLevel = 4
	else
		local addonver, addonsub, addonbeta = strsplit(".", at.Cache.Version.number)
		local settingver, settingsub, settingbeta = strsplit(".", JMMDB.Version)
		if tonumber(addonver) > tonumber(settingver) then at.Cache.Version.ResetLevel = at.Cache.Version.ResetLevel +4 end
		if tonumber(addonsub) > tonumber(settingsub) then at.Cache.Version.ResetLevel = at.Cache.Version.ResetLevel +2 end
		if tonumber(addonbeta) > tonumber(settingbeta) then at.Cache.Version.ResetLevel = at.Cache.Version.ResetLevel +1 end
	end

	--Resets
	if at.Cache.Version.ResetLevel > 0 then--Beta
		if at.Cache.Version.ResetLevel > 1 then--Sub
			if at.Cache.Version.ResetLevel > 3 then--Version
				JMMDB = {
					Version = at.Cache.Version.number,
					PetBattle = true,--emulates pet battle as combat
					Arrow = true,
					Anchors = {},
					Addons = {},
					Map = {
						alpha		= 0.7,
						scale		= 1,
						combat		= true,
						fade		= 0,
						visible		= true,
						shape		= 2,--index value for list of shapes
						Date		= true,
						Coords = {
							enabled		= true,
							alpha		= 1,
							spacer		= ":",
							Font = {--this also applies to the time
								name	= [[Fonts\FRIZQT__.TTF]],
								size	= 12,
								style	= 'OUTLINE',--outline is used since there are no backgrounds in this addon
							},
						},
						Zone = {
							alpha	= 1,
							combat	= false,
							fade	= .75,
							Font = {
								name	= [[Fonts\FRIZQT__.TTF]],
								size	= 16,
								style	= 'OUTLINE',
							},
						},
						Icons = {
							alpha	= 1,
							scale	= 1,
							combat	= true,
						},
						Compass = {
							enabled = true,
							alpha	= 1,
							scale	= 1,
							combat	= false,
							fade	= 0,
						},
						Blip = {
							blink = false,
							style = nil,
							refresh = 2,
						},
					},
					Blobs = {
						Quests = {
							Enabled = true,
							Alpha = 0.25,
							Style = "Quests",
						},
						Archaeology = {
							Enabled = true,
							Alpha = 0.25,
							Style = "Archaeology",
						},
						QuestsFilter = "NONE",
						Quality = 0,
					},
					Profile = {},
					Addons = {
						ignore = at.Adjunct.Addons.ignore,--I've decided to let the user define other ignored addons
						capture = at.Adjunct.Addons.capture,
						ButtonFrame = {
							growth = 'DOWN',
							wrap = 0,
							direction = 'RIGHT',--used only for wrapping; this should be a direction adjacent to growth

							Fade = {
								enabled = true,
								value = 13,
							},
							Position = {
								point = 'TOPRIGHT',
								relative = 'TOPRIGHT',
								x = 0,
								y = -30,
							},
						},
					},
				}
			end--full version reset
			JMMDB.Addons = {
				ignore = at.Adjunct.Addons.ignore,--I've decided to let the user define other ignored addons
				capture = at.Adjunct.Addons.capture,
				ButtonFrame = {
					growth = 'DOWN',
					wrap = 0,
					direction = 'RIGHT',--used only for wrapping; this should be a direction adjacent to growth

					Fade = {
						enabled = true,
						value = 13,
					},
					Position = {
						point = 'TOPRIGHT',
						relative = 'TOPRIGHT',
						x = 0,
						y = -30,
					},
				},
			}
			JMMDB.Map.Coords.Font = {
				name	= [[Fonts\FRIZQT__.TTF]],
				size	= 12,
				style	= 'OUTLINE',
			}
			JMMDB.Map.Zone.Font = {
				name	= [[Fonts\FRIZQT__.TTF]],
				size	= 16,
				style	= 'OUTLINE',
			}
			JMMDB.Map.Zone.point = 'UP'
		end--sub version reset
		JMMDB.Anchors = nil
	end--beta version reset
	JMMDB.Version = at.Cache.Version.number
	if not JMMCharacterDB then
		JMMCharacterDB = {
			Profile = {
				current = 'self',
			},
			Settings = at.Abet:CopyTable(JMMDB),--this will hold the profile settings
		}
		JMMCharacterDB.Settings.Profile = nil
		JMMCharacterDB.Settings.Map.Blip.style = nil--reset to allow profession detection
		at.Cache.Version.ResetLevel = -1--identifies that the movers need to be shown
	end

	local profile = JMMCharacterDB.Profile.current
	if profile == 'default' then
		at.Settings = at.Abet:CopyTable(JMMDB)
	elseif profile == 'self' then
		at.Settings = at.Abet:CopyTable(JMMCharacterDB.Settings)
	else
		at.Settings = at.Abet:CopyTable(JMMDB.Profile[profile])
	end
	at.Miniblobs:Unpack( at.Settings.Blobs )
	at.Settings.Profile = nil

end
--PLAYER_LOGIN; fires once
function at.Events:PLAYER_LOGIN(...)
	self:UnregisterEvent('PLAYER_LOGIN')
	at.Events['PLAYER_LOGIN'] = nil
	local name, ref
	for x, v in pairs(at.Cache.Minimap.Icons) do--FrameStrata/alpha work around
		ref = _G[x]-- at.Cache.Mnimap.Icons.List[x]
		ref:ClearAllPoints()
		ref:SetParent(at.Core)
		ref:SetPoint('CENTER', Minimap, v)
	end
	at.Adjunct:SetFonts()
	at.Adjunct:FindMinimapChildren()
	at.Adjunct:CreateAddonButtonFrame(at.Settings.Addons.ButtonFrame)
end
--PLAYER_LOGOUT; fires once
function at.Events:PLAYER_LOGOUT(...)
	--save to disk
	if JMMDB ~= nil and JMMCharacterDB ~= nil then
		at.Settings.Blobs = at.Miniblobs:Pack()
		local profile = JMMCharacterDB.Profile.current
		if profile == 'default' then
			local sub = JMMDB.Profile
			JMMDB = at.Settings
			JMMDB.Profile = sub
		elseif profile == 'self' then
			JMMCharacterDB.Settings = at.Settings
		else
			JMMDB.Profile[profile] = at.Settings
		end
	end
end
--PLAYER_ENTERING_WORLD; fires after the player sees a loading screen
function at.Events:PLAYER_ENTERING_WORLD(...)
	at.Movers:Create()

	if at.Settings.Map.Coords.enabled then
		at.Adjunct.CoordsFrame:SetAlpha(at.Settings.Map.Coords.alpha)
		at.Adjunct.CoordsFrame:Show()
	end

	--change TomTomLites arrow if JMM's is enabled
	if IsAddOnLoaded("TomTomLite") then
		local ref = TomTomLiteArrow
		at.Cache.Arrow.arrow = ref.arrow:GetTexture()
		at.Cache.Arrow.glow = ref.glow:GetTexture()
		if at.Settings.Arrow then
			ref.arrow:SetTexture(at.Cache.Arrow.JMMArrow)
			ref.glow:SetTexture(at.Cache.Arrow.JMMGlow)
		end
	end

	--odd Minimap behavior
	Minimap:Hide()
	Minimap:Show()
	at.Minimap:Update('update')
	at.Minimap:UpdateTrackingBlip(at.Settings.Map.Blip.style)
	at.Compass:Update()
	at.Core:SetAlpha(at.Settings.Map.Icons.alpha)


	at.Events['PLAYER_ENTERING_WORLD'] = function(...)
		--odd Minimap behavior
		Minimap:Hide()
		at.Minimap:Update('update')
		Minimap:Show()
		--at.Compass:Update()
	end
end

--Actually call the event functions
at.Core:SetScript("OnEvent", function(self, event, ...)
	at.Events[event](self, ...);
end)
