local addonName, at = ...

at.Movers = {
	list = {
		'MinimapCluster',--'DurabilityFrame', 'WatchFrame'
		'TicketStatusFrame',--ticket
		'VehicleSeatIndicator',--vehicle
		'WorldStateAlwaysUpFrame',--pvpstate
		'BuffFrame',--buffs
		'UIErrorsFrame',
		--'QueueStatusFrame',
		--'TimeManagerFrame',
	},
	anchors = {
		'QueueStatusFrame',
	},
}

local silhouette = UIParent:CreateTexture(nil, 'BACKGROUND')
silhouette:SetTexture([[Interface\BUTTONS\WHITE8X8]])
silhouette:SetAlpha(0.45)
silhouette:Hide()

local function OnDragStart(self)
	local ref = self:GetParent()
	if ref:IsVisible()then
		silhouette:SetParent(ref)
		silhouette:ClearAllPoints()
		silhouette:SetAllPoints(ref)
		silhouette:Show()
	end
	ref:StartMoving()
end

local function OnDragStop(self)
	silhouette:Hide()
	self:GetParent():StopMovingOrSizing()
end

function at.Movers:CreateAnchor(name, moveFrame)
	--still a mover despite it being an anchor
	local frame = CreateFrame('Button', addonName .. name .. "Mover", UIParent)
	frame:EnableMouse(true)
	frame:SetClampedToScreen(true)
    frame:RegisterForDrag("LeftButton", "RightButton")
	frame:SetScript('OnDragStart', function(self) self:StartMoving() end)
	frame:SetScript('OnDragStop', function(self)
		self:StopMovingOrSizing()
	end)
	frame:SetScript("OnHide", function(self)
		self:StopMovingOrSizing()
	end)

	local texture = frame:CreateTexture(nil, 'BORDER')
	texture:SetTexture([[Interface\Buttons\UI-Panel-Button-Up]])
	texture:SetAllPoints()
	texture:SetTexCoord(0, 0.625, 0, 0.6875)

	local string = frame:CreateFontString(nil, 'ARTWORK', 'GameFontNormal')
	string:SetPoint('CENTER', 1, 0)
	string:SetText(name)
	frame:SetHeight(string:GetStringHeight() + 8)
	frame:SetWidth(string:GetStringWidth() + 16)
	frame:SetMovable(true)
	frame:ClearAllPoints()
	local sub = at.Settings.Anchors[name]
	if not sub then
		sub = {
			point = 'CENTER',
			--[2] = UIParent,
			relate = 'CENTER',
			x = 0,
			y = 0,
		}
		at.Settings.Anchors[name] = sub
	end
	frame:SetPoint(sub.point, nil, sub.relate, sub.x, sub.y)

	frame:Hide()
	return frame
end

function at.Movers:CreateButton(name, moveFrame)
	local frame = CreateFrame('Button', addonName .. name .. "Mover", moveFrame)
	frame:EnableMouse(true)
	frame:SetClampedToScreen(true)
	frame:SetFrameStrata('MEDIUM')
    frame:RegisterForDrag("LeftButton", "RightButton")
	frame:SetScript('OnDragStart', OnDragStart)
	frame:SetScript('OnDragStop', OnDragStop)
	frame:SetScript("OnHide", function(self) moveFrame:StopMovingOrSizing() end)

	local texture = frame:CreateTexture(nil, 'BORDER')
	texture:SetTexture([[Interface\Buttons\UI-Panel-Button-Up]])
	texture:SetAllPoints()
	texture:SetTexCoord(0, 0.625, 0, 0.6875)

	local string = frame:CreateFontString(nil, 'ARTWORK', 'GameFontNormal')
	string:SetPoint('CENTER', 1, 0)
	string:SetText(name)
	frame:SetHeight(string:GetStringHeight() + 8)
	frame:SetWidth(string:GetStringWidth() + 16)
	if name == 'MinimapCluster' then
		frame:SetPoint('BOTTOM', moveFrame)
	else
		frame:SetPoint('TOP', moveFrame)
	end

	frame:Hide()
	return frame
end

--this will clean up after itself
function at.Movers:Create()
	local ref
	for x, v in pairs(at.Movers.list) do
		ref = _G[v]
		ref:SetMovable(true)
		ref:SetClampedToScreen(true)
		ref:SetUserPlaced(true)
		ref = at.Movers:CreateButton(v, ref)
		if at.Cache.Version.ResetLevel > 3 or at.Cache.Version.ResetLevel == -1 then
			ref:Show()
		end
	end
	for x, v in pairs(at.Movers.anchors) do
		ref = _G[v]
		ref:SetMovable(true)
		ref:SetClampedToScreen(true)
		ref = at.Movers:CreateAnchor(v, ref)
		--at.Movers.anchors[v] = ref
		if at.Cache.Version.ResetLevel > 3 or at.Cache.Version.ResetLevel == -1 then
			ref:Show()
		end
	end
	self.CreateAnchor = nil
	self.CreateButton = nil
	self.Create = nil
end

function at.Movers:Show()
	local ref
	for x, v in pairs(at.Movers.list) do
		_G[addonName .. v .. 'Mover']:Show()
	end
	for x, v in pairs(at.Movers.anchors) do
		_G[addonName .. v .. 'Mover']:Show()
	end
end
function at.Movers:Hide()
	local ref
	for x, v in pairs(at.Movers.list) do
		_G[addonName .. v .. 'Mover']:Hide()
	end
	for x, v in pairs(at.Movers.anchors) do
		_G[addonName .. v .. 'Mover']:Hide()
	end
end
